//  Database Engine Wrapper
//  Copyright  1994-1996
//  B. W. Bush
//  All rights reserved
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// Modifyed by Yura Bidus for use with OWL NExt
// 

// Record Reading Example


#include "owlpch.h"

#include <owlfx/link.h>
#include <owl/private/owllink.h>

_OWLFX_USING_NAMESPACE

#define DBASE_FILE "dew_test.db"

int OwlMain(int , TCHAR* [])
{
    // Start the engine.

    TDewEngine engine;

    // Open a database.

    TDewDatabase database;

    // Try to extract the data from the table "DEW_TEST.DB".

    try {

        // Create a table for the table and find its record buffer.

        TDewTable table(database, DBASE_FILE);
        TDewRecord& record = table.GetCurRecord();

        // Print the attributes (field) names.

        cout << "Reading (Name, Date, Amount) . . . " << endl;

        // Print all of the tuples (records).

        for (table.SetToBegin(); table.Advance(); ) {
            string name;
            TDate date;
            float amount;
            record.GetField("Name", name);
            record.GetField("Date", date);
            const bool blank = record.GetField("Amount", amount);
            date.SetPrintOption(TDate::Numbers);
            cout << "\t(\"" << name << "\", " << date << ", ";
            if (blank)
                cout << "<blank>";
            else
                cout << amount;
            cout << ")" << endl;
        }

        cout << "Data successfully read." << endl;

    } catch(const TDewException& exception) {

        // Report an error reading the data.

        cerr << "Data could not be read: " << exception << endl;

    }

    return 0;
}