//  Database Engine Wrapper
//  Copyright  1994-1996
//  B. W. Bush
//  All rights reserved
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// Modifyed by Yura Bidus for use with OWL NExt
// 
// DEW Example

#include "owlpch.h"
#include <owlfx/link.h>
#include <owl/private/owllink.h>

_OWLFX_USING_NAMESPACE

#define DBASE_FILE "dew_test.db"

int OwlMain(int , TCHAR* [])
{
  // Start the engine.
  TDewEngine engine;

  // Open a database.
  TDewDatabase database;

  // Try to create the table "DEW_TEST.DB".
  try {
      // Make first field ten characters long called "Name", the second field a date called "Date", and the third field a number called "Amount".
      TDewFldDesc fields[3];
      fields[0] = TDewFldDesc(1, "Name", fldZSTRING, 0, 10);
      fields[1] = TDewFldDesc(2, "Date", fldDATE);
      fields[2] = TDewFldDesc(3, "Amount", fldFLOAT);

      // Make "Name" + "Date" the primary key.

      DBIKEY keyFld;
      keyFld[0] = 1;
      keyFld[1] = 2;
      TDewIdxDesc index(keyFld, 2, NULL, 0, NULL, TRUE);

      // Make the table have three fields and one index.

      TDewCrTblDesc descriptor("DEW_TEST.DB", 3, fields, 1, &index);

      // Create the table.

      database.CreateTable(descriptor);

      cout << "Table \"DEW_TEST.DB\" creation succeeded." << endl;

  } catch(const TDewException& exception) {

      // Report an error creating the table.

      cerr << "Table \"DEW_TEST.DB\" creation failed: " << exception << endl;

  }

  cout << endl;

  // Try to put some data in the table "DEW_TEST.DB".

  try {

      // Create a table for the table and find its record buffer.

      TDewTable table(database, "DEW_TEST.DB");
      TDewRecord& record = table.GetCurRecord();

      // Append the data.

      table.InitRecord();
      record.SetField("Name", "Arnold");
      record.SetField("Date", TDate());
      record.SetField("Amount", 0.00);
      table.AppendRecord();

      table.InitRecord();
      record.SetField(1, "Bruce");
      record.SetField(2, TDate(31, 10, 1994));
      record.SetField(3, 5.00);
      table.AppendRecord();

      table.InitRecord();
      record.SetField(1, "Charles");
      record.SetField(2, TDate(31, 10, 1994));
      record.SetBlank(3);
      table.AppendRecord();

      table.InitRecord();
      record.SetField(1, "Charles");
      record.SetField(2, TDate(1, 11, 1994));
      record.SetField(3, -2.50);
      table.AppendRecord();

      cout << "Records successfully added." << endl;

  } 
	catch(TDewException& x) {
		// Report an error appending the data.
    cerr << "Records could not be added: " << x << endl;
  }

  cout << endl;

  // Try to put some data in the table "DEW_TEST.DB" that violates its primary key.

  try {

      // Create a table for the table and find its record buffer.

      TDewTable table(database, "DEW_TEST.DB");
      TDewRecord& record = table.GetCurRecord();

      // Append some data that has the same key as data previously added.

      table.InitRecord();
      record.SetField("Name", "Arnold");
      record.SetField("Date", TDate());
      record.SetField("Amount", 10.00);
      try {
          table.AppendRecord();
          cerr << "Key violation not detected." << endl;
      } 
			catch (TDewIntegrityException& x) {
				if (x.Result() == DBIERR_KEYVIOL)
					cout << "Key violation successfully detected." << endl;
        else
					throw;
      }

  } 
	catch(TDewException& x) {
    // Report a failure to catch the key violation.
    cerr << "Key violation not detected." << endl;
  }
  cout << endl;
  // Try to extract the data from the table "DEW_TEST.DB".
  try {
		// Create a table for the table and find its record buffer.
    TDewTable table(database, "DEW_TEST.DB");
    TDewRecord& record = table.GetCurRecord();

    // Print the attributes (field) names.
    cout << "Reading (Name, Date, Amount) . . . " << endl;

		// Print all of the tuples (records).
		for (table.SetToBegin(); table.Advance(); ) {
      string name;
      TDate date;
      float amount;
      record.GetField("Name", name);
      record.GetField("Date", date);
      const BOOL blank = record.GetField("Amount", amount);
      date.SetPrintOption(TDate::Numbers);
      cout << "\t(\"" << name << "\", " << date << ", ";
      if (blank)
          cout << "<blank>";
      else
          cout << amount;
      cout << ")" << endl;
		}
		cout << "Data successfully read." << endl;

	}
	catch(TDewException& exception) {
		// Report an error reading the data.
    cerr << "Data could not be read: " << exception << endl;
  }
  cout << endl;

	// Try to search for a key value in "DEW_TEST.DB".
  try {
      // Create a table for the table and find its record buffer.
      TDewTable table(database, "DEW_TEST.DB");
      TDewRecord& record = table.GetCurRecord();

      // Create a record buffer for the key.
      TDewKey key(table);
      TDewRecord keyRecord(table);
      keyRecord.SetField("Name", "Bruce");
      table.ExtractKey(key, keyRecord);

      // Search for the key.

      cout << "Searching for (\"Bruce\", *, *) . . ." << endl;
      table.GetRecordForKey(key, 1);
      string name;
      TDate date;
      float amount;
      record.GetField("Name", name);
      record.GetField("Date", date);
      record.GetField("Amount", amount);
      date.SetPrintOption(TDate::Numbers);
      cout << "\t(\"" << name << "\", " << date << ", " << amount << ")" << endl;

      cout << "Search successfully completed." << endl;

  } catch(const TDewException& exception) {

      // Report a failure of the search.

      cerr << endl << "Key could not be found: " << exception << endl;

  }

  cout << endl;

 // Try to set and retrieve a bookmark in "DEW_TEST.DB".

  try {

      // Create a table for the table and find its record buffer.

      TDewTable table(database, "DEW_TEST.DB");
      TDewRecord& record = table.GetCurRecord();

      // Create a bookmark.

      TDewBookMark bookMark(table);

      table.Advance();
      table.Advance();

      string name;
      record.GetField("Name", name);
      cout << "Setting book mark at \"" << name << "\"" << endl;
      table.GetBookMark(bookMark);

      // Go to the book mark.

      table.SetToBegin();
      cout << "Moving to book mark . . . \"";
      table.SetToBookMark(bookMark);
      string name2;
      record.GetField("Name", name2);
      cout << name2 << "\"" << endl;

      cout << "Bookmark successfully found." << endl;

  } catch(const TDewException& exception) {

      // Report a failure of the search.

      cerr << endl << "Bookmark could not be found: " << exception << endl;

  }

  cout << endl;

  // Try to delete the table "DEW_TEST.DB".

  try {

      // Delete the table.

      database.DeleteTable("DEW_TEST.DB");

      cout << "Table successfully deleted." << endl;

  }
  catch(const TDewException& exception) {
    // Report an error deleting the table.
    cerr << "Table could not be deleted: " << exception << endl;
 }
  return 0;
}
